package iaa.map.client.iseephone;

import java.util.*;
import java.io.*;
import java.net.*;
import java.rmi.*;

//import iaa.map.GUI.*;
import iaa.map.server.*;

/**
 * Simple class for visual display of a MapDataSet. It exists to test server functionality,
 * and to provide a very simple example of server interaction and MapDataSet use.
 */
public class IseePhoneClient 
{
    private MapServer server;

    private byte[] cameraKey;

    //  private java.util.List path = null;

    static final String DEFAULT_HOST = "66.93.61.121";

    public static String speechDirectory = "speech";

    public static String[] speech = 
    {
	"intro",  "Hello. Welcome to I see.",
	"menu1",  "Press one to get random directions.\n Press two to restart.\n Press zero to exit.",
	"exit",         "Goodbye.\n Disconnect now.",
	"error",        "That was not a valid Fuckeeng Option.\n What.",
	"precompPath",  "This is a precomputed path.",
	"wait4path",    "Computing path.\n Please wait.",
	"timeout",      "Session timed out."
    };

    private List wayPoints = new ArrayList();

    public static class WayPoint {
	LatLonPosition pos;
	String name;

	public WayPoint(LatLonPosition pos, String name) {
	    this.pos = pos;
	    this.name = name;
	}
    }
    
    public IseePhoneClient() throws Exception
    {
	this(null, true);
    }
    public IseePhoneClient( String rmiService, boolean local ) 
	throws Exception
    {
	computeSpeechSamples();
	
	// connect to server
	// either start a local MapServer or call a remote server
	if (local) {
	    server = new ServerNode();
	} else {
	    System.out.println("looking up RMI service:" + rmiService);
	    try {
		server = (MapServer)Naming.lookup(rmiService);
	    } catch (RemoteException e) {
		System.out.println("RMI call failed: " + e);
	    }
	}

	// query the server about its layers and remember the camera layer's key
	System.out.println("Server has the following layers:");
	MapLayerDescription[] layerInfo = server.getLayerInfo();
	cameraKey = null;
	for (int i = 0; i < layerInfo.length; i++) {
	    System.out.println(i + ": " + layerInfo[i]);
	    if (layerInfo[i] != null && layerInfo[i].getName().equals("Cameras")) {
		cameraKey = layerInfo[i].getKey();
		System.out.println("cameraKey = " + cameraKey);
	    }
	}

	// set up waypoints
	wayPoints.add(new WayPoint(new LatLonPosition(40.746098f,-74.00498f), "Eye beam gallery. 540 West 21st Street"));
	wayPoints.add(new WayPoint(new LatLonPosition(40.7252f, -73.99658f), "New Museum of Contemporary Art. 583 Broadway"));
	
    }

    private void computeSpeechSamples()
    {
	TextToSpeech tts = new TextToSpeech(speechDirectory);

	for (int i = 0; i < speech.length; i += 2) {
	    String name = speech[i];
	    String value = speech[i+1];

	    File file = new File(speechDirectory + "/" + name + ".rmd");
	    if (!file.exists()) {
		System.out.print("Computing \"" + name + "\"...");
		tts.textToRMD(value, name);
		System.out.println("done");
	    }
	}
    }

    /**
     *  Throws together a CPQuery that requests data from all layers.
     */
    public static CenterPointQuery makeQuery( LatLonPosition center, float scale, 
					      int height, int width) 
    {
	CenterPointQuery q = new CenterPointQuery(center, scale, height, width);
	q.setQueryAllLayers(true);
	return q;
    }

    public void doVoiceInterface()
    {
	VoiceModemInterface vmi = new VoiceModemInterface();

	vmi.open();
	System.out.println("opened VoiceModem interface.");

	boolean donePgm = false;
	while (!donePgm) {
	    vmi.initModem();
	    System.out.println("modem initialized");
		
	    boolean doneCall = false;
	    while (!doneCall) {
		System.out.println("waiting for ring...");
		vmi.waitForRing();
		System.out.println("got ring.");
		
		boolean doneMenu = false;
		vmi.startTransmitMode();
		vmi.transmitFile("speech/intro.rmd");
		while (!doneMenu) {
		    vmi.transmitFile("speech/menu1.rmd");
		    String result = vmi.waitForKeys(1, 15000);
		    if (result != null && result.length() >= 1) {
			if (result.startsWith("1")) {
			    vmi.transmitFile("speech/wait4path.rmd", false);
			    String pathFile = getRandomPath();
			    vmi.transmitFile(speechDirectory + "/" + pathFile + ".rmd");
			} else if (result.startsWith("2")) {
			    // just start over
			} else if (result.startsWith("0")) {
			    doneMenu = true;
			    doneCall = true;
			} else {
			    vmi.transmitFile("speech/error.rmd");
			}
		    } else {
			vmi.transmitFile("speech/timeout.rmd");
			doneMenu = true;
			doneCall = true;			
		    }		    
		}
	    }

	    vmi.transmitFile("speech/exit.rmd");
	    vmi.endTransmitMode();
	    
	    System.out.print("hanging up...");
	    vmi.hangup();
	    System.out.println("done.");
	}
	vmi.close();
	System.out.println("closed VoiceModem interface.");
    }
   
    public String getRandomPath()
    {
	String pathFile = "directions";

	try {

	    // plan a between two waypoints
	    WayPoint wp1 = (WayPoint)wayPoints.get(0);
	    WayPoint wp2 = (WayPoint)wayPoints.get(1);
	    
	    LatLonPosition start = wp1.pos;
	    LatLonPosition end = wp2.pos;
	    
	    PathQuery pq = new PathQuery(start, end);
	    pq.addLayer(cameraKey);
	    MapDataSet data = null;
	    try {
		data = server.query(pq);
	    } catch	 (RemoteException e) {
		System.out.println("RMI call failed: " + e);
	    }
	    
	    TextToSpeech tts = new TextToSpeech(speechDirectory);
	    if (data != null) {
		List path = data.getPath();
		
		ReadiblePath rpath = new ReadiblePath(path);
		System.out.println("got rpath: " + rpath);
		String rmdFile = tts.textToRMD(rpath.toString(), pathFile);
	    } else {
		String rmdFile = tts.textToRMD("error computing path! sorry! try again.", pathFile);
	    }
	} catch (Exception e) {	    
	    System.out.println("got exception: " + e);
	    e.printStackTrace();
	    TextToSpeech tts = new TextToSpeech(speechDirectory);
	    String rmdFile = tts.textToRMD("aaw Shiit! Got an exception. Here it is. " + e, pathFile);
	}

	return pathFile;
    }

    /**
     *  Runs the iSee phone client. 
     *  <br> the -l switch runs and uses a local server.
     *  <br> the -n rmi://host/serviceName option indicates a remote server to connect to.
     */
    public static void main(String[] args) 
	throws Exception
    {
	boolean local = false;

	//	BookmarkHolder bookmarks = BookmarkHolder.open();
    
	String rmiService = "rmi://" + DEFAULT_HOST + "/" + ServerNode.DEFAULT_SERVICE_NAME;
	
	// parse the command line args
	for (int i = 0; i < args.length; i++) {
	    if (args[i].equals("-l")) {
		local = true;
	    }
	    ///// service lookup args
	    if (args[i].equals("-n") && (i+1) < args.length) {
		rmiService = args[++i];
	    }	    
	}

	IseePhoneClient pc = new IseePhoneClient( rmiService, local );


	pc.doVoiceInterface();
    }
}
