package iaa.map.client.iseephone;

import java.util.*;
import java.io.*;

import iaa.map.GUI.*;
import iaa.map.server.*;

/**
 * converts text to wav files. 
 * 
 */
public class TextToSpeech
{
    public String workingDirectory = "/tmp";

    public String ttsBinary = "bin/text2wave";

    public String pvftoolsBinDir = "bin";

    public int ttsSampleRate = 8000;
    public int rmdSampleRate = 7200;
    public float ttsAmplitudeScale = 4.0f;

    public TextToSpeech()
    {
    }

    public TextToSpeech( String workingDirectory ) 
    {
	this.workingDirectory = workingDirectory;
    }

    public String textToWav( String text, String filename )
    {
	if (filename == null) filename = "directions";

	String textFile = workingDirectory + "/" + filename + ".txt";
	String wavFile  = workingDirectory + "/" + filename + ".wav";

	// write the text file to be converted
	try {
	    PrintWriter out = new PrintWriter( new FileWriter(textFile) );
	    out.println(text);
	    out.close();
	} catch (IOException e) {
	    System.err.println("could not write to output file " + textFile + ": " + e);
	}
       
	// do the conversion
	String[] pgm = {ttsBinary, 
			"-o " + wavFile + 
			" -otype wav" + 
			" -F " + ttsSampleRate + 
			" -scale " + ttsAmplitudeScale +
			" " + textFile};
	Runtime rt = Runtime.getRuntime();
	try {
	    Process child = rt.exec(pgm);
	    child.waitFor();
	} catch (IOException e) {
	    System.err.println("error: " + e);
	} catch (InterruptedException e) {
	    System.err.println("error: " + e);
	}

	return wavFile;
    }

    public String wavToRMD( String wavFile, String filename )
    {
	String pvfFile    = workingDirectory + "/" + filename + ".pvf";
	String pvfSpdFile = workingDirectory + "/" + filename + ".spd.pvf";
	String rmdFile    = workingDirectory + "/" + filename + ".rmd";

	String wav2pvfBinary  = pvftoolsBinDir + "/wavtopvf";
	String pvfspeedBinary = pvftoolsBinDir + "/pvfspeed";
	String pvf2rmdBinary  = pvftoolsBinDir + "/pvftormd";

	// convert wav file to pvf format
	String step1 = wav2pvfBinary + " " + wavFile + " " + pvfFile;
	// convert pvf file to 7200kHz sample rate
	String step2 = pvfspeedBinary + " -s 7200 " + pvfFile + " " + pvfSpdFile;
	// convert pvf to Rockwell rmd format used by modem (with a header)
	String step3 = pvf2rmdBinary + " Rockwell 4 " + pvfSpdFile + " " + rmdFile;

	Runtime rt = Runtime.getRuntime();
	try {
	    Process child1 = rt.exec(step1);
	    child1.waitFor();
	    Process child2 = rt.exec(step2);
	    child2.waitFor();
	    Process child3 = rt.exec(step3);
	    child3.waitFor();
	} catch (IOException e) {
	    System.err.println("error: " + e);
	} catch (InterruptedException e) {
	    System.err.println("error: " + e);
	}

	// clean up the intermediate files
	File f;
	try {
	    f = new File(wavFile);
	    f.delete();
	    f = new File(pvfFile);
	    f.delete();
	    f = new File(pvfSpdFile);
	    f.delete();
	} catch (Exception e) {
	    System.err.println("failed to delete some intermediate files: " + e);
	}

	return rmdFile;
    }

    public String textToRMD( String text, String filename )
    {
	String wavFile = textToWav( text, filename );
	return wavToRMD( wavFile, filename );	
    }
}
